/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.document.index.persistence.impl.db;

import com.topcoder.db.connectionfactory.ConfigurationException;
import com.topcoder.db.connectionfactory.DBConnectionException;
import com.topcoder.db.connectionfactory.DBConnectionFactory;
import com.topcoder.db.connectionfactory.DBConnectionFactoryImpl;
import com.topcoder.db.connectionfactory.UnknownConnectionException;

import com.topcoder.document.index.CollectionIndex;
import com.topcoder.document.index.DocumentIndex;
import com.topcoder.document.index.persistence.IndexPersistence;
import com.topcoder.document.index.persistence.IndexPersistenceException;
import com.topcoder.document.index.persistence.impl.PersistenceConfigurationException;
import com.topcoder.document.index.wordsource.WordSourceId;

import com.topcoder.util.config.ConfigManager;
import com.topcoder.util.config.UnknownNamespaceException;

import java.util.List;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map;
import java.util.Set;


/**
 * <p>
 * This is an abstraction of a typical data base driven persistence contract. This provides
 * for getting a connection from the DBConnectionFactory as well as specifying transaction control.
 * This hides nicely the intricacies of configuration that would be used by all derived classes.
 * </p>
 * <p>
 * Thread Safety: This class is not thread-safe.
 * </p>
 *
 *
 * @author AleaActaEst, TCSDEVELOPER
 * @version 1.0
 */
public abstract class AbstractDBIndexPersistence implements IndexPersistence {
    /**
     * <p>Static connection name property string.</p>
     */
    private static final String CONNECTION_NAME_PROPERTY = "ConnectionName";

    /**
     * <p>Static connection factory name property string.</p>
     */
    private static final String CONNECTION_FACTORY_CLASS_NAME_PROPERTY = "ConnectionFactoryClassName";

    /**
     * <p>Static connection factory namespace property string.</p>
     */
    private static final String CONNECTION_FACTORY_NAMESPACE_PROPERTY = "ConnectionFactoryNamespace";

    /**
     * <p>SQL to insert a document record.</p>
     */
    static final String INSERT_DOCUMENT =
        "INSERT INTO DOCUMENT (DOCUMENT_ID, LOCALE, SOURCE_IDENTITY, CLASS_NAME, USE_COUNT) VALUES(?, ?, ?, ?, 0)";

    /**
     * <p>SQL to select document record.</p>
     */
    static final String SELECT_DOCUMENT = "SELECT LOCALE, SOURCE_IDENTITY, CLASS_NAME, USE_COUNT"
        + " FROM DOCUMENT WHERE DOCUMENT_ID=?";

    /**
     * <p>SQL to select all documents.</p>
     */
    static final String SELECT_ALL_DOCUMENT = "SELECT DOCUMENT_ID, LOCALE, SOURCE_IDENTITY, CLASS_NAME, USE_COUNT"
        + " FROM DOCUMENT";

    /**
     * <p>SQL to select all dele.</p>
     */
    static final String SELECT_DELIMITERS =
        "SELECT DOCUMENT_ID, DELIMITER_ENTRY FROM DELIMITER WHERE DOCUMENT_ID IN ";



    /**
     * <p>SQL to insert a delimiter record.</p>
     */
    static final String INSERT_DELIMITER =
        "INSERT INTO DELIMITER (DOCUMENT_ID, DELIMITER_ENTRY) VALUES(?, ?)";

    /**
     * <p>SQL to select document records.</p>
     */
    static final String SELECT_DELIMITER = "SELECT DELIMITER_ENTRY FROM DELIMITER WHERE DOCUMENT_ID=?";


    /**
     * <p>SQL to select document collection records.</p>
     */
    static final String SELECT_COLLECTION = "SELECT COLLECTION_ID "
        + " FROM DOCUMENT_COLLECTION WHERE COLLECTION_ID=?";



    /**
     * <p>SQL to insert a word record in the fast way.</p>
     */
    static final String INSERT_WORD_FAST =
        "INSERT INTO WORD_DATA (DOCUMENT_ID, WORD_DATA) VALUES(?, ?)";

    /**
     * <p>SQL to select a document record.</p>
     */
    static final String SELECT_WORD_FAST = "SELECT WORD_DATA FROM WORD_DATA WHERE DOCUMENT_ID=?";

    /**
     * <p>SQL to delete document record.</p>
     */
    static final String DELETE_DOCUMENT = "DELETE FROM DOCUMENT WHERE DOCUMENT_ID=?";

    /**
     * <p>SQL to delete document record.</p>
     */
    static final String DELETE_DOCUMENT_DELIMITER = "DELETE FROM DELIMITER WHERE DOCUMENT_ID=?";

    /**
     * <p>SQL to delete document record.</p>
     */
    static final String DELETE_WORD_FAST = "DELETE FROM WORD_DATA WHERE DOCUMENT_ID=?";

    /**
     * <p>SQL to update the document use count.</p>
     */
    static final String UPDATE_DOCUMENT_USE_COUNT = "UPDATE DOCUMENT SET USE_COUNT=USE_COUNT+? WHERE DOCUMENT_ID=?";

    /**
     * <p>SQL to insert the collection index.</p>
     */
    static final String INSERT_COLLECTION_INDEX = "INSERT INTO DOCUMENT_COLLECTION(COLLECTION_ID) VALUES (?)";

    /**
     * <p>SQL to insert the DOCUMENT_COLLECTION_DOCUMENT_XREF.</p>
     */
    static final String INSERT_COLLECTION_INDEX_DOCUMENT_INDEX
        = "INSERT INTO DOCUMENT_COLLECTION_DOCUMENT_XREF(COLLECTION_ID, DOCUMENT_ID) VALUES (?, ?)";

    /**
     * <p>SQL to delete DOCUMENT_COLLECTION_DOCUMENT_XREF record.</p>
     */
    static final String DELETE_COLLECTION_DOCUMENT_XREF
        = "DELETE FROM DOCUMENT_COLLECTION_DOCUMENT_XREF WHERE COLLECTION_ID=?";

    /**
     * <p>SQL to delete document colletion record.</p>
     */
    static final String DELETE_DOCUMENT_COLLECTION
        = "DELETE FROM DOCUMENT_COLLECTION WHERE COLLECTION_ID=?";


    /**
     * <p>SQL to select all documents.</p>
     */
    static final String SELECT_COLLECTION_DOCUMENTS
        = "SELECT DOCUMENT.DOCUMENT_ID, DOCUMENT.LOCALE, DOCUMENT.SOURCE_IDENTITY, DOCUMENT.CLASS_NAME"
        + "    FROM DOCUMENT LEFT OUTER JOIN DOCUMENT_COLLECTION_DOCUMENT_XREF"
        + "        ON DOCUMENT_COLLECTION_DOCUMENT_XREF.DOCUMENT_ID=DOCUMENT.DOCUMENT_ID"
        + "    WHERE DOCUMENT_COLLECTION_DOCUMENT_XREF.COLLECTION_ID=?";

    /**
     * <p>
     * This represents the connection factory from which we will obtain a pre-configured
     * connection for our data base access.
     * This is initialized in one of the constructors and once initialized cannot be changed.
     * Cannot be null.
     * <p>
     */
    private final DBConnectionFactory connectionFactory;

    /**
     * <p>
     * This represents a connection name (an alias) that is used to fetch a connection
     * instance from the connection factory.
     * This is initialized in one of the constructors and once initialized cannot be changed.
     * Can be null or an empty string, upon which it will try to use the default connection.
     * </p>
     */
    private final String connectionName;

    /**
     * <p>
     * An abstract constructor which will populate the connectionFactory and connectionName
     * information from configuration.
     * </p>
     * <p>
     * The given namespace should contain 'connectionFactoryClassName', 'connectionFactoryNamespace' and
     * 'ConnectionName' properties. If 'connectionFactoryClassName' property is missing,
     * 'com.topcoder.db.connectionfactory.DBConnectionFactoryImpl' will be used.
     * If 'connectionFactoryNamespace' property is missing, the default constructor will be used.
     * If 'ConnectionName' property is missing, default connection will be used.
     * </p>
     *
     *
     * @param namespace The namespace for connection-related properties.
     * @throws IllegalArgumentException if namespace param is null or empty.
     * @throws PersistenceConfigurationException if there are configuration issues encountered
     */
    protected AbstractDBIndexPersistence(String namespace)
        throws PersistenceConfigurationException {
        DBIndexPersistenceHelper.validateNotNullOrEmpty(namespace, "namespace");

        String connFactoryClassName = null;
        String connFactoryNamespace = null;
        ConfigManager configManager = ConfigManager.getInstance();

        // get configuration properties
        try {
            connFactoryNamespace = configManager.getString(namespace,
                    CONNECTION_FACTORY_NAMESPACE_PROPERTY);
            // ConnectionFactoryNamespace property is required, throw an exception if it's missing
            if (connFactoryNamespace == null || connFactoryNamespace.trim().length() == 0) {
                throw new PersistenceConfigurationException(
                        CONNECTION_FACTORY_NAMESPACE_PROPERTY + " property is missing");
            }
            connectionName = configManager.getString(namespace, CONNECTION_NAME_PROPERTY);

            connFactoryClassName = configManager.getString(namespace,
                    CONNECTION_FACTORY_CLASS_NAME_PROPERTY);

        } catch (UnknownNamespaceException une) {
            throw new PersistenceConfigurationException(namespace + " namespace does not exist", une);
        }

        // if ConnectionFactoryClassName property is missing, use DBConnectionFactoryImpl calss as default
        if (connFactoryClassName == null || connFactoryClassName.trim().length() == 0) {
            try {
                connectionFactory = new DBConnectionFactoryImpl(connFactoryNamespace);
            } catch (UnknownConnectionException uce) {
                throw new PersistenceConfigurationException(
                        "failed to create DBConnectionFactoryImpl instance.", uce);
            } catch (ConfigurationException ce) {
                throw new PersistenceConfigurationException(
                        "failed to create DBConnectionFactoryImpl instance.", ce);
            }
        } else {
            connectionFactory = createConnectionFactory(connFactoryClassName, connFactoryNamespace);
        }

    }

    /**
     * Creates a DBConnectionFactory instance with given class name and namespace.
     *
     * @param className the DBConnectionFactory class name
     * @param namespace the namespace used to create the DBConnectionFactory instance
     * @return the created DBConnectionFactory instance.
     * @throws PersistenceConfigurationException if failed to create the DBConnectionFactory instance.
     */
    private DBConnectionFactory createConnectionFactory(String className, String namespace)
        throws PersistenceConfigurationException {

        // get the class
        Class connFactoryClass = null;
        try {
            connFactoryClass = Class.forName(className);
        } catch (ClassNotFoundException cnfe) {
            throw new PersistenceConfigurationException(className + " class is not found.", cnfe);
        }

        // create the DBConnectionFactory instance
        DBConnectionFactory factory = null;
        try {
            Constructor ctor = connFactoryClass.getConstructor(new Class[] {String.class});
            factory = (DBConnectionFactory) ctor.newInstance(new Object[] {namespace});
        } catch (NoSuchMethodException e) {
            throw new PersistenceConfigurationException("Cannot get the constructor.", e);
        } catch (IllegalAccessException e) {
            throw new PersistenceConfigurationException(
                "Failed to create " + className + " instance", e);
        } catch (InstantiationException e) {
            throw new PersistenceConfigurationException(
                "Failed to create " + className + " instance", e);
        } catch (InvocationTargetException e) {
            throw new PersistenceConfigurationException(
                "Failed to create " + className + " instance", e);
        } catch (SecurityException e) {
            throw new PersistenceConfigurationException("Failed to access " + className, e);
        } catch (ClassCastException e) {
            throw new PersistenceConfigurationException("The class is not a DB connection factory.", e);
        }

        return factory;
    }

    /**
     * <p>
     * An abstract constructor which will populate the connectionFactory and connectionName
     * information from input parameters.
     * </p>
     *
     * @param connectionFactory DBConnectionFactory instance used to generate connections.
     * @param connectionName The name of the connection used for persistence operations.
     * @throws IllegalArgumentException if connectionFactory is null.
     */
    protected AbstractDBIndexPersistence(DBConnectionFactory connectionFactory, String connectionName) {
        DBIndexPersistenceHelper.validateNotNull(connectionFactory, "connectionFactory");
        this.connectionFactory = connectionFactory;
        this.connectionName = connectionName;
    }

    /**
     * <p>
     * Returns a new connection to be used for persistence. This method will obtain
     * the connection from the connectionFactory using the connection name alias.
     * If connection name is not specified, a defualt connection will be used.
     * Note: auto-commit is disabled.
     * </p>
     *
     * @return connection got from connection factory
     * @throws IndexPersistenceException If unable to obtain a Connection.
     */
    protected Connection getConnection() throws IndexPersistenceException {
        Connection connection = null;

        try {
            // try to obtain the connection
            if ((connectionName == null) || (connectionName.trim().length() == 0)) {
                connection = connectionFactory.createConnection();
            } else {
                connection = connectionFactory.createConnection(connectionName);
            }

            connection.setAutoCommit(false);
        } catch (DBConnectionException dbce) {
            throw new IndexPersistenceException("Unable to create the connection from factory.", dbce);
        } catch (SQLException sqle) {
            dispose(connection);
            throw new IndexPersistenceException("Failed to disable auto-commit.", sqle);
        }

        return connection;
    }

    /**
     * <p>
     * Starts a transaction on the passed in connection.
     * </p>
     *
     * @param connection The Connection taking part in the transaction.
     * @throws IndexPersistenceException If an error occurs during this process, or if
     * the Connection does not support transactions.
     * @throws IllegalArgumentException if connection is null
     */
    protected void beginTransaction(Connection connection) throws IndexPersistenceException {
        DBIndexPersistenceHelper.validateNotNull(connection, "connection");

        // Currently we do nothing here
    }

    /**
     * <p>
     * Commits a transaction on the passed in connection. The connection is closed after commit.
     * </p>
     *
     * @param connection The Connection taking part in the transaction.
     * @throws IndexPersistenceException If an error occurs during this process, or if
     * the Connection does not support transactions.
     * @throws IllegalArgumentException if connection is null
     */
    protected void commitTransaction(Connection connection)
        throws IndexPersistenceException {
        DBIndexPersistenceHelper.validateNotNull(connection, "connection");

        try {
            connection.commit();
        } catch (SQLException sqle) {
            throw new IndexPersistenceException("Failed to commit the transaction.", sqle);
        } finally {
            dispose(connection);
        }
    }

    /**
     * <p>
     * Rolls back the transaction. The connection is closed after rollback.
     * </p>
     *
     * @param connection The Connection taking part in the transaction.
     * @throws IndexPersistenceException If an error occurs during this process, or if
     * the Connection does not support transactions.
     * @throws IllegalArgumentException if connection is null
     */
    protected void rollbackTransaction(Connection connection)
        throws IndexPersistenceException {
        DBIndexPersistenceHelper.validateNotNull(connection, "connection");

        try {
            connection.rollback();
        } catch (SQLException sqle) {
            throw new IndexPersistenceException("Failed to rollback the transaction.", sqle);
        } finally {
            dispose(connection);
        }
    }





    /**
     * Add the document record and it's delimiters.
     *
     * @param conn the database connection
     * @param documentIndex the DocumentIndex to insert
     * @param documentId the id of the document
     * @throws IndexPersistenceException if failed to insert the document
     */
    void addDocumentIndexCommon(Connection conn, DocumentIndex documentIndex, String documentId)
            throws IndexPersistenceException {

        // first, insert the document record

        // retrieve id, local, className from documentIndex
        // and serialize source identity
        WordSourceId wordSourceId = documentIndex.getWordSourceId();
        String local = documentIndex.getLocale().toString();
        String className = wordSourceId.getSourceClassName();
        byte[] bytes = DBIndexPersistenceHelper.serializeObject(new Object[] {wordSourceId.getSourceIdentity()});

        executeUpdate(conn, INSERT_DOCUMENT, new Object[] {documentId, local, bytes, className});

        // second, insert the delimiters records
        String[] delimiters = documentIndex.getWordSourceId().getDelimiters();
        for (int i = 0; i < delimiters.length; ++i) {
            executeUpdate(conn, INSERT_DELIMITER, new Object[] {documentId, delimiters[i]});
        }

    }


    /**
     * <p>Updates the use count value for document index with specified WordSourceId.
     * Use count is the integer value stored in the persistence along with every
     * document index. This value is equal to the number of collections, that contain
     * a particular document.</p>
     *
     * @param wordSourceId WordSourceId of document index use count of which to update,
     * should not be null
     * @param value 1 or -1 to update
     * @throws IllegalArgumentException if wordSourceId is null
     * @throws IndexPersistenceException when fails to increase document index
     * use count in the persistence
     */
    void updateDocumentUseCount(WordSourceId wordSourceId, int value)
        throws IndexPersistenceException {
        DBIndexPersistenceHelper.validateNotNull(wordSourceId, "wordSourceId");

        String documentId = DBIndexPersistenceHelper.calculateId(wordSourceId);

        Connection conn = getConnection();
        beginTransaction(conn);
        try {
            // throw an exception when decreacing when use count is 0
            if (value == -1) {
                int count = getDocumentUseCount(conn, documentId);
                if (count == 0) {
                    throw new IndexPersistenceException("The use count is zero, cannot decrease");
                }
            }

            executeUpdate(conn, UPDATE_DOCUMENT_USE_COUNT, new Object[] {new Integer(value), documentId});
        } catch (IndexPersistenceException e) {
            rollbackTransaction(conn);
            throw e;
        }
        commitTransaction(conn);
    }

    /**
     * <p>
     * Return the count of how many different groups this document is a member of.
     * </p>
     *
     * @return the count of how many different groups this document is a member of.
     * @param wordSourceId WordSourceId of document index use count of which to update,
     * should not be null
     * @throws IllegalArgumentException if wordSourceId is null
     * @throws IndexPersistenceException when fails to increase document index
     * use count in the persistence
     */
    int getDocumentUseCountCommon(WordSourceId wordSourceId) throws IndexPersistenceException {
        DBIndexPersistenceHelper.validateNotNull(wordSourceId, "wordSourceId");

        // calculate the id of the document
        String documentId = DBIndexPersistenceHelper.calculateId(wordSourceId);

        Connection conn = getConnection();
        beginTransaction(conn);


        int count = 0;
        try {
            count = getDocumentUseCount(conn, documentId);

        } catch (IndexPersistenceException e) {
            rollbackTransaction(conn);
            throw e;
        }
        commitTransaction(conn);
        return count;
    }

    /**
     *
     * @param conn
     * @param documentId
     * @return
     * @throws IndexPersistenceException
     */
    int getDocumentUseCount(Connection conn, String documentId) throws IndexPersistenceException {
        PreparedStatement ps = null;
        try {
            ps = prepareStatement(conn, SELECT_DOCUMENT, new Object[] {documentId});
            ResultSet rs = ps.executeQuery();
            // if no such document id, throw an exception
            if (!rs.next()) {
                throw new IndexPersistenceException("The specified doucment does not exist.");
            }
            return rs.getInt(4);
        } catch (SQLException sqle) {
            throw new IndexPersistenceException("failed to get the use count", sqle);
        } finally {
            dispose(ps);
        }
    }

    /**
     * <p>
     * Return set of WordSourceId of documents that are specified by given query and parameters.
     * </p>
     *
     * @param conn the database connection
     * @param update the query get required documents
     * @param parameters the query parameters
     *
     * @return a set of WordSourceId of documents
     * @throws IndexPersistenceException when failed to get the document ids
     */
    Set getIndexedDocuments(Connection conn, String query, Object[] parameters)
        throws IndexPersistenceException {

        // Note: this method use 2 sql queries to get all required WordSourceId instances.
        // This is much faster then join the document and delimiter table

        PreparedStatement ps = null;
        List documentIds = new ArrayList();
        Map localeMap = new HashMap();
        Map classNameMap = new HashMap();
        Map sourceIdMap = new HashMap();
        Map delimiterMap = new HashMap();

        try {
            // The first query will retrieve all reqired documentIds, locales, class names, and source ids
            // These fileds are stored in corresponding maps
            ps = prepareStatement(conn, query, parameters);
            ResultSet rs = ps.executeQuery();
            while (rs.next()) {
                // retrieve fields
                String documentId = rs.getString(1);
                String localeString = rs.getString(2);
                Locale locale = new Locale(localeString.substring(0,2), localeString.substring(3,5));

                Object sourceId = DBIndexPersistenceHelper.deserializeObject(rs.getBytes(3));
                String className = rs.getString(4);

                // update the field maps
                documentIds.add(documentId);
                localeMap.put(documentId, locale);
                classNameMap.put(documentId, className);
                sourceIdMap.put(documentId, sourceId);
                delimiterMap.put(documentId, new ArrayList());
            }

            // return an empty set if there is no document
            if (documentIds.size() == 0) {
                return new HashSet();
            }

            // The second query will retrieve all delimites
            ps = prepareStatement(conn, SELECT_DELIMITERS + createValues(documentIds), new Object[0]);
            rs = ps.executeQuery();
            while (rs.next()) {
                String documentId = rs.getString(1);
                String delimiter = rs.getString(2);
                List delimiters = (List) delimiterMap.get(documentId);
                delimiters.add(delimiter);
            }


        } catch (SQLException sqle) {
            throw new IndexPersistenceException("failed to get the documents", sqle);
        } finally {
            dispose(ps);
        }


        // at last, we populate the WordSourceId instances.
        Set wordSourceIds = new HashSet();
        for (Iterator it = documentIds.iterator(); it.hasNext();) {
            String id = (String) it.next();
            WordSourceId wordSourceId = null;
            try {
                wordSourceId = new WordSourceId(
                    sourceIdMap.get(id),
                    (String) classNameMap.get(id),
                    (String[]) ((List) delimiterMap.get(id)).toArray(new String[0]),
                    (Locale) localeMap.get(id));

            } catch (IllegalArgumentException iae) {
                throw new IndexPersistenceException("failed to create the WordSourceId instance", iae);
            }
            wordSourceIds.add(wordSourceId);
        }

        return wordSourceIds;
    }

    /**
     * <p>Remove a document index with given WordSourceId from the persistence.
     * When the document index is not found, or use count is not zero, exception
     * is thrown.</p>
     *
     * @param wordSourceId WordSourceId of document index to remove, should not be null
     * @param deleteQueries the queries to delete document.
     * @throws IllegalArgumentException when WordSourceId is null
     * @throws IndexPersistenceException when document index is not found in the
     * peristence, or error happens when trying to remove it
     */
    void removeDocumentIndexCommon(WordSourceId wordSourceId, String[] deleteQueries)
        throws IndexPersistenceException {
        DBIndexPersistenceHelper.validateNotNull(wordSourceId, "wordSourceId");

        Connection conn = getConnection();

        try {
            // get the id of the document
            String documentId = DBIndexPersistenceHelper.calculateId(wordSourceId);

            // get the use count, and throw an exception if it's not zero
            int count = getDocumentUseCount(conn, documentId);
            if (count != 0) {
                throw new IndexPersistenceException("use count is not zero.");
            }
            // delete the rows from given tables.
            for (int i = 0; i < deleteQueries.length; ++i ) {
                executeUpdate(conn, deleteQueries[i], new Object[] {documentId});
            }

        } catch (IndexPersistenceException e) {
            rollbackTransaction(conn);
            throw e;
        }

        commitTransaction(conn);
    }

    /**
     * <p>
     * Return set of WordSourceId of documents that have document index persisted.
     * </p>
     *
     * @return set of WordSourceId of documents that have document index persisted.
     * @throws IndexPersistenceException when fails
     */
    Set getIndexedDocumentsCommon() throws IndexPersistenceException {

        Connection conn = getConnection();
        beginTransaction(conn);

        Set documents = null;
        try {
            documents = getIndexedDocuments(conn, SELECT_ALL_DOCUMENT, new Object[0]);
        } catch (IndexPersistenceException e) {
            rollbackTransaction(conn);
            throw e;
        }
        commitTransaction(conn);
        return documents;

    }




    /**
     * <p>Update the given DocumentIndex in the persistence.</p>
     *
     * @param documentIndex WordSource representing the document to be indexed, should
     * not be null
     * @throws IllegalArgumentException when documentIndex is null
     * @throws IndexPersistenceException when fails to add DocumentIndex to
     * the persistence
     */
    void updateCollectionIndexCommon(CollectionIndex collectionIndex) throws IndexPersistenceException {
        DBIndexPersistenceHelper.validateNotNull(collectionIndex, "collectionIndex");

        Connection conn = getConnection();
        beginTransaction(conn);

        try {
            if (isEmpty(conn, SELECT_COLLECTION, new Object[] {collectionIndex.getId()})) {
                throw new IndexPersistenceException("the given collection does not exits.");
            }
            removeCollectionIndex(conn, collectionIndex.getId());
            addCollectionIndex(conn, collectionIndex);
        } catch (IndexPersistenceException e) {
            rollbackTransaction(conn);
            throw e;
        }

        commitTransaction(conn);
    }

    /**
     * <p>Add the given DocumentIndex to the persistence.</p>
     *
     * @param documentIndex WordSource representing the document to be indexed, should
     * not be null
     * @throws IllegalArgumentException when documentIndex is null
     * @throws IndexPersistenceException when fails to add DocumentIndex to
     * the persistence
     */
    void addCollectionIndexCommon(CollectionIndex collectionIndex) throws IndexPersistenceException {
        DBIndexPersistenceHelper.validateNotNull(collectionIndex, "collectionIndex");

        Connection conn = getConnection();
        beginTransaction(conn);

        try {
            addCollectionIndex(conn, collectionIndex);
        } catch (IndexPersistenceException e) {
            rollbackTransaction(conn);
            throw e;
        }

        commitTransaction(conn);
    }

    /**
     * <p>Add the given DocumentIndex to the persistence.</p>
     *
     * @param documentIndex WordSource representing the document to be indexed, should
     * not be null
     * @param conn the database connection
     * @throws IndexPersistenceException when fails to add DocumentIndex to
     * the persistence
     */
    private void addCollectionIndex(Connection conn, CollectionIndex collectionIndex)
        throws IndexPersistenceException {
        executeUpdate(conn, INSERT_COLLECTION_INDEX, new Object[] {collectionIndex.getId()});

        for (Iterator it = collectionIndex.getAllDocumentIds().iterator(); it.hasNext();) {
            WordSourceId wordSourceId = (WordSourceId) it.next();

            String documentId = DBIndexPersistenceHelper.calculateId(wordSourceId);

            executeUpdate(conn, INSERT_COLLECTION_INDEX_DOCUMENT_INDEX,
                    new Object[] {collectionIndex.getId(), documentId});
        }
    }

    /**
     * <p>Remove document collection index with specified identifier from the persistence.
     *
     * @param collectionId Identifer of document  collection index to remove, should
     * not be null or empty string
     * @throws IllegalArgumentException when collectionId is null or empty string
     * @throws IndexPersistenceException when fails to remove CollectionInde
     * with given identifer
     */
    void removeCollectionIndexCommon(String collectionId) throws IndexPersistenceException {
        DBIndexPersistenceHelper.validateNotNullOrEmpty(collectionId, "collectionId");

        Connection conn = getConnection();
        beginTransaction(conn);

        try {
            removeCollectionIndex(conn, collectionId);
        } catch (IndexPersistenceException e) {
            rollbackTransaction(conn);
            throw e;
        }

        commitTransaction(conn);
    }


    /**
     * <p>Remove document collection index with specified identifier from the persistence.</p>
     *
     * @param collectionId Identifer of document collection index to remove
     * @param conn the database connection
     * @throws IndexPersistenceException when fails to remove CollectionInde
     * with given identifer
     */
    private void removeCollectionIndex(Connection conn, String collectionId) throws IndexPersistenceException {
        // remove all associated record in DOCUMENT_COLLECTION_DOCUMENT_XREF table and DOCUMENT_COLLECTION table
        executeUpdate(conn, DELETE_COLLECTION_DOCUMENT_XREF, new Object[] {collectionId});
        executeUpdate(conn, DELETE_DOCUMENT_COLLECTION, new Object[] {collectionId});
    }



    /**
     * Prepares the statement for given sql query.
     *
     * @param conn the database connection
     * @param query the sql query in string format
     * @param parameters the query parameters
     * @return the PreparedStatement instance
     * @throws SQLException if failed to create the statement
     */
    PreparedStatement prepareStatement(Connection conn, String query, Object[] parameters)
        throws SQLException {

        // create the PreparedStatement instance
        PreparedStatement ps = null;
        ps = conn.prepareStatement(query);

        // initialize parameters
        for (int i = 0; i < parameters.length; ++i) {
            if (parameters[i] instanceof String) {
                ps.setString(i + 1, (String) parameters[i]);
            } else {
                ps.setObject(i + 1, parameters[i]);
            }
        }
        return ps;
    }


    /**
     * Executes the given sql update with the given parameters.
     *
     * @param conn the database connection
     * @param update the sql update to execute
     * @param parameters the update parameters
     * @return the number of rows affected
     * @throws IndexPersistenceException if failed to execute the update
     */
    int executeUpdate(Connection conn, String update, Object[] parameters) throws IndexPersistenceException {
        // execute the update and close the PreparedStatement finally
        PreparedStatement ps = null;
        try {
            ps = prepareStatement(conn, update, parameters);
            return ps.executeUpdate();
        } catch (SQLException sqle) {
            throw new IndexPersistenceException("failed to execute the update", sqle);
        } finally {
            dispose(ps);
        }
    }

    /**
     * Checks whether the given query returns an empty result set.
     *
     * @param conn the database connection
     * @param query the sql query in string format
     * @param parameters the query parameters
     * @return whether the given query returns an empty result set.
     * @throws SQLException if failed to create the statement
     */
    boolean isEmpty(Connection conn, String query, Object[] parameters) throws IndexPersistenceException {
        // execute the query and close the PreparedStatement finally
        PreparedStatement ps = null;
        try {
            ps = prepareStatement(conn, query, parameters);
            ResultSet rs = ps.executeQuery();
            return !rs.next();
        } catch (SQLException sqle) {
            throw new IndexPersistenceException("failed to execute the query", sqle);
        } finally {
            dispose(ps);
        }
    }


    /**
     * Dispose Connection.
     *
     * @param conn the connection.
     */
    void dispose(Connection conn) {
        if (conn != null) {
            try {
                conn.close();
            } catch (SQLException e) {
                // ignore
            }
        }
    }

    /**
     * Dispose PreparedStatement.
     *
     * @param ps the prepared statement.
     */
    void dispose(PreparedStatement ps) {
        if (ps != null) {
            try {
                ps.close();
            } catch (SQLException e) {
                // ignore
            }
        }
    }

    /**
     * Gets the first delimiter in alphabetical order in given delimiters.
     *
     * @param delimiters the delimiters
     * @return the first delimiter in alphabetical order
     */
    String chooseDelimiter(String[] delimiters) {
        String delimiter = delimiters[0];
        for (int i = 1; i < delimiters.length; ++i) {
            if (delimiter.compareTo(delimiters[i]) > 0) {
                delimiter = delimiters[i];
            }
        }
        return delimiter;
    }

    /**
     * Create a string containing given values.
     * The string is like ('value1', 'value2', 'value3')
     * @param values the values
     * @return a string like ('value1', 'value2', 'value3')
     */
    public static String createValues(Collection values) {
        StringBuffer ret = new StringBuffer();
        ret.append('(');
        for (Iterator itr = values.iterator(); itr.hasNext();) {
            if (ret.length() > 1) {
                ret.append(", ");
            }
            ret.append("'").append(itr.next()).append("'");
        }
        ret.append(')');
        return ret.toString();
    }

}
