/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.document.index.persistence.impl.db;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Arrays;

import com.topcoder.document.index.persistence.IndexPersistenceException;
import com.topcoder.document.index.wordsource.WordSourceId;


/**
 * <p>
 * Defines some helper methods used in this component.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
final class DBIndexPersistenceHelper {
    /**
     * The name of md5 digest algorithm.
     */
    private static final String MD5_DIGEST = "MD5";
    /**
     * <p>
     * This private constructor prevents the creation of new instance.
     * </p>
     */
    private DBIndexPersistenceHelper() {
    }

    /**
     * <p>
     * Validates the value of a variable with type <code>Object</code>.
     * The value cannot be <code>null</code>.
     * </p>
     *
     * @param value value of the variable.
     * @param name name of the variable.
     *
     * @throws IllegalArgumentException if <code>value</code> is <code>null</code>.
     */
    static void validateNotNull(Object value, String name) {
        if (value == null) {
            throw new IllegalArgumentException(name + " cannot be null.");
        }
    }


    /**
     * <p>
     * Validates the value of a string variable. The value cannot be <code>null</code> and empty string.
     * </p>
     *
     * @param value value of the variable.
     * @param name name of the variable.
     *
     * @throws IllegalArgumentException if <code>value</code> is null or empty string.
     */
    static void validateNotNullOrEmpty(String value, String name) {
        validateNotNull(value, name);
        if (value.trim().length() == 0) {
            throw new IllegalArgumentException(name + " cannot be empty string.");
        }
    }


    /**
     * Calculates a md5 digest base on given object.
     *
     * @param object the object to calculate
     * @return a md5 digest in hex string format
     * @throws IndexPersistenceException if failed to calculate the id
     */
    static String calculateId(WordSourceId wordSourceId) throws IndexPersistenceException {

        // Note:
        // The order of the delimiters could not be reserved after we insert it to database
        // in order to get the same id for those wordSourceIds, we sort the delimiters before serialization
        String[] delimiters = wordSourceId.getDelimiters();
        Arrays.sort(delimiters);

        // serialize the content of wordSourceId
        byte[] bytes = serializeObject(new Object[] {
                wordSourceId.getSourceIdentity(),
                wordSourceId.getSourceClassName(),
                wordSourceId.getSourceLocale().toString(),
                delimiters});

        // get the MD5 digest algorithm
        MessageDigest messageDigest = null;
        try {
            messageDigest = MessageDigest.getInstance(MD5_DIGEST);
        } catch (NoSuchAlgorithmException nsae) {
            throw new IndexPersistenceException("Failed to get MessageDigest", nsae);
        }

        // calculate the digest
        messageDigest.update(bytes);
        byte[] digest = messageDigest.digest();

        return bytesToHexString(digest);
    }

    /**
     * Serializes the given object to a byte array.
     *
     * @param object the object to serialize
     * @return a byte array containing the serialized object
     * @throws IndexPersistenceException if failed to serialize the id
     */
    static byte[] serializeObject(Object[] objects) throws IndexPersistenceException {
        // create a ByteArrayOutputStream to store the object in bytes.
        ByteArrayOutputStream bytes = new ByteArrayOutputStream();
        ObjectOutputStream oos = null;
        try {
            // using ObjectOutputStream to serialize the object
            oos = new ObjectOutputStream(bytes);
            for (int i = 0; i < objects.length; ++i) {
                oos.writeObject(objects[i]);
            }
        } catch (IOException ioe) {
            throw new IndexPersistenceException("Failed to serialize the object", ioe);
        } finally {
            try {
                oos.close();
            } catch (IOException e) {
                // ignore
            }
        }

        return bytes.toByteArray();
    }

    /**
     * Deserializes the given object to a byte array.
     *
     * @param object the object to serialize
     * @return a byte array containing the serialized object
     * @throws IndexPersistenceException if failed to serialize the id
     */
    static Object deserializeObject(byte[] bytes) throws IndexPersistenceException {
        ObjectInputStream ois = null;
        try {
            ois = new ObjectInputStream(new ByteArrayInputStream(bytes));
            return ois.readObject();
        } catch (IOException ioe) {
            throw new IndexPersistenceException("Failed to serialize the object", ioe);
        }  catch (ClassNotFoundException cnfe) {
            throw new IndexPersistenceException("Failed to serialize the object", cnfe);
        } finally {
            try {
                ois.close();
            } catch (IOException e) {
                // ignore
            }
        }
    }

    /**
     * <p>
     * Convertes the byte array to a hex string.
     * </p>
     *
     * @param bytes the bytes to process
     *
     * @return the hex string of the bytes
     */
    static String bytesToHexString(byte[] bytes) {
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < bytes.length; ++i) {
            // convert byte to Hex string
            String hex = Integer.toHexString(bytes[i] & 0xff);
            if (hex.length() == 1) {
                // if the length of the hex string is 1, add '0' before it
                sb.append('0');
            }
            sb.append(hex);
        }
        return sb.toString();
    }

}




