/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.document.index.persistence.impl.db;

import com.topcoder.db.connectionfactory.DBConnectionFactory;

import com.topcoder.document.index.CollectionIndex;
import com.topcoder.document.index.DocumentIndex;
import com.topcoder.document.index.persistence.IndexPersistenceException;
import com.topcoder.document.index.persistence.impl.PersistenceConfigurationException;
import com.topcoder.document.index.wordsource.WordSourceId;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.CollationKey;
import java.text.Collator;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;


/**
 * <p>This is an implementation of the AbstractDBIndexPersistence that deals with a fast,
 * CLOB based, normalized data based schema. This is provided for a user how is concerned
 * about speed of the execution. By storing all the word and word position data in a
 * single CLOB structure we are achieving a 4000% speed increase over a comparable
 * solution that uses thousands of records to store words and related positions.
 * <p>
 * Thread Safety: This class is not thread-safe.
 * </p>
 *
 * @author AleaActaEst, TCSDEVELOPER
 * @version 1.0
 */
public class FastAccessDBIndexPersistence extends AbstractDBIndexPersistence {

    /**
     * <p>
     * An simple constructor which will populate the connectionFactory and
     * connectionName information from configuration.
     * </p>
     *
     * @param namespace The namespace for connection-related properties.
     * @throws IllegalArgumentException if namespace param is null or empty.
     * @throws PersistenceConfigurationException if there are configuration issues encountered
     */
    public FastAccessDBIndexPersistence(String namespace)
        throws PersistenceConfigurationException {
        super(namespace);
    }

    /**
     * <p>
     * A simple constructor which will populate the connectionFactory and
     * connectionName information from input parameters.
     * </p>
     *
     * @param connectionFactory DBConnectionFactory instance used to generate connections.
     * @param connectionName The name of the connection used for persistence operations.
     * @throws IllegalArgumentException if connectionFactory is null.
     */
    public FastAccessDBIndexPersistence(DBConnectionFactory connectionFactory, String connectionName) {
        super(connectionFactory, connectionName);
    }

    /**
     * <p>Add the given DocumentIndex to the persistence.</p>
     *
     * @param documentIndex WordSource representing the document to be indexed, should
     * not be null
     * @throws IllegalArgumentException when documentIndex is null
     * @throws IndexPersistenceException when fails to add DocumentIndex to
     * the persistence
     */
    public void addDocumentIndex(DocumentIndex documentIndex) throws IndexPersistenceException {
        DBIndexPersistenceHelper.validateNotNull(documentIndex, "documentIndex");

        Connection conn = getConnection();

        try {
            // get the id of the document
            String documentId = DBIndexPersistenceHelper.calculateId(documentIndex.getWordSourceId());

            // add the document and delimiter records, this is the common part of simple and fast implementation
            addDocumentIndexCommon(conn, documentIndex, documentId);

            // add the words in fast way
            insertWordFast(conn, documentIndex, documentId);

        } catch (IndexPersistenceException e) {
            rollbackTransaction(conn);
            throw e;
        }

        commitTransaction(conn);
    }



    /**
     * <p>Retrieve a DocumentIndex with the specified WordSourceId from the persistence.
     * Return null, if DocumentIndex with the given WordSourceId is not found in the
     * persistence.</p>
     *
     * @return retrieved DocumentIndex, or null if document index with given WordSourceId
     * is not found in the persistence
     * @param wordSourceId WordSourceId of document to retrieve, should not be null
     * @throws IllegalArgumentException when wordSourceId is null
     * @throws IndexPersistenceException when fails to retreive document index
     * with the specified WordSourceId
     */
    public DocumentIndex getDocumentIndex(WordSourceId wordSourceId) throws IndexPersistenceException {
        DBIndexPersistenceHelper.validateNotNull(wordSourceId, "wordSourceId");

        DocumentIndex documentIndex = null;

        Connection conn = getConnection();
        try {
            documentIndex = getDocumentIndexFast(conn, wordSourceId);
        } catch (IndexPersistenceException e) {
            rollbackTransaction(conn);
            throw e;
        }

        commitTransaction(conn);

        return documentIndex;
    }

    /**
     * <p>Retrieve a DocumentIndex with the specified WordSourceId from the persistence in fast way.
     * Return null, if DocumentIndex with the given WordSourceId is not found in the
     * persistence.</p>
     *
     * @return retrieved DocumentIndex, or null if document index with given WordSourceId
     * is not found in the persistence
     * @param wordSourceId WordSourceId of document to retrieve, should not be null
     * @throws IndexPersistenceException when fails to retreive document index
     * with the specified WordSourceId
     */
    private DocumentIndex getDocumentIndexFast(Connection conn, WordSourceId wordSourceId)
        throws IndexPersistenceException {

        String documentId = DBIndexPersistenceHelper.calculateId(wordSourceId);

        // gets the word data from database
        String wordData = null;
        PreparedStatement ps = null;
        try {
            // check if the given wordSourceId exists in database
            if (isEmpty(conn, SELECT_DOCUMENT, new Object[] {documentId})) {
                return null;
            }

            // select the word data
            ps = prepareStatement(conn, SELECT_WORD_FAST, new Object[] {documentId});
            ResultSet rs = ps.executeQuery();

            // if there is no word data, throws an exception
            if (!rs.next()) {
                throw new IndexPersistenceException("no word data associated to given document");
            }

            wordData = rs.getString(1);

            // if there is more than one word data record, throws an exception
            if (rs.next()) {
                throw new IndexPersistenceException("more than one word data associated to given document");
            }
        } catch (SQLException sqle) {
            throw new IndexPersistenceException("failed to select the words", sqle);
        } finally {
            dispose(ps);
        }

        // converts word data string to a map and return
        Map words = parseWords(wordData, wordSourceId);
        return new DocumentIndex(wordSourceId, words);
    }


    /**
     * Parses the word data string to a word map.
     *
     * @param wordData the string containing word data
     * @param wordSourceId the wordSourceId
     * @return a map containing all word and their postions.
     * @throws IndexPersistenceException if failed to parse the word data
     */
    private Map parseWords(String wordData, WordSourceId wordSourceId) throws IndexPersistenceException {

        // **NOTE**
        // In order to ensure the same delimiter is used in both encoding and decoding.
        // we choose the smallest delimiter in alphabetical order.
        // Using the first delimiter is not safe, since the order of the delimiters
        // can't be reserved after we insert delimiters to database.
        String delimiter = this.chooseDelimiter(wordSourceId.getDelimiters());

        Collator collator = Collator.getInstance(wordSourceId.getSourceLocale());

        Map words = new HashMap();
        int pos = 0;
        String word = null;
        List positions = null;
        while (true) {
            // gets the next token between two delimiters
            String nextToken = null;
            int newPos = wordData.indexOf(delimiter, pos);
            if (newPos != -1) {
                nextToken = wordData.substring(pos, newPos);
            }
            pos = newPos + delimiter.length();

            // if there is no more token, simply exit
            if (nextToken == null) {
                break;
            }

            if (nextToken.length() == 0) {

                // throw an exception if the data is incorrect
                if (word == null || positions.size() == 0) {
                    throw new IndexPersistenceException("Invalid word data in database");
                }
                // all position for current word are retrieved, put it to the word map.
                words.put(collator.getCollationKey(word), positions);
                word = null;
            } else if (word == null) {
                // find a new word, reset the position list
                word = nextToken;
                positions = new ArrayList();
            } else {
                // find a postion, pasre it and add to the position list
                try {
                    positions.add(Integer.valueOf(nextToken));
                } catch (NumberFormatException nfe) {
                    throw new IndexPersistenceException("Invalid position in word data", nfe);
                }

            }

        }
        return words;
    }

    /**
     * Inserts a record to word_data table.
     *
     * @param conn the database connection
     * @param documentIndex the DocumentIndex containing the words
     * @param documentId the document id
     * @throws IndexPersistenceException if failed to insert the word
     */
    void insertWordFast(Connection conn, DocumentIndex documentIndex, String documentId)
        throws IndexPersistenceException {

        Map words = documentIndex.getWords();

        // **NOTE**
        // In order to ensure the same delimiter is used in both encoding and decoding.
        // we choose the smallest delimiter inalphabetical order.
        // Using the first delimiter is not safe, since the order of the delimiters
        // can't be reserved after we insert delimiters to database.
        String delimiter = chooseDelimiter(documentIndex.getWordSourceId().getDelimiters());

        // converts the words and positions to a string
        // e.g. if delimiter is \n the converted string is like:
        // word1\npos1\npos2\n\nword2\pos3\n\n;
        // refer to CS for more details
        StringBuffer wordData = new StringBuffer();
        // converts each words
        for (Iterator wordIterator = words.entrySet().iterator(); wordIterator.hasNext();) {
            Map.Entry entry = (Map.Entry) wordIterator.next();
            String word = ((CollationKey) entry.getKey()).getSourceString();
            List positions = (List) entry.getValue();
            wordData.append(word).append(delimiter);

            // converts each postions
            for (Iterator positionIterator = positions.iterator(); positionIterator.hasNext();) {
                Integer postion = (Integer) positionIterator.next();
                wordData.append(postion).append(delimiter);
            }

            wordData.append(delimiter);
        }
        executeUpdate(conn, INSERT_WORD_FAST, new Object[] {documentId, wordData.toString().getBytes()});

    }


    /**
     * <p>Remove a document index with given WordSourceId from the persistence.
     * When the document index is not found, or use count is not zero, exception
     * is thrown.</p>
     *
     * @param wordSourceId WordSourceId of  document index to remove, should not be null
     * @throws IllegalArgumentException when WordSourceId is null
     * @throws IndexPersistenceException when document index is not found in the
     * peristence, or error happens when trying to remove it
     */
    public void removeDocumentIndex(WordSourceId wordSourceId) throws IndexPersistenceException {
        removeDocumentIndexCommon(wordSourceId,
                new String[] {DELETE_WORD_FAST, DELETE_DOCUMENT_DELIMITER, DELETE_DOCUMENT});
    }



    /**
     * <p>
     * Store document collection index in the persistence.
     * </p>
     *
     * @param collectionIndex CollectionIndex to store, should not be null
     * @throws IllegalArgumentException if collectionIndex is null
     * @throws IndexPersistenceException if fails to create and store document
     * collection index
     */
    public void addCollectionIndex(CollectionIndex collectionIndex) throws IndexPersistenceException {
        addCollectionIndexCommon(collectionIndex);
    }

    /**
     * <p>Retrieve a document collection index with specified identifier.
     * Return null when collection with specified identifier is not found,
     * throw exception when collection index cannot be retrieved in any
     * other case.
     *
     * Implementation note:
     *
     * Please consult the CS section 1.4.3.2.
     * NOTE: When creating the CollectionIndex object using the three-param constructor,
     * note that the owner param will
     * be set to null.
     * </p>
     *
     * @return CollectionIndex with specified identifer
     * @param collectionId identifer of document collection index to retrieve, should not
     * be null or empty string
     * @throws IllegalArgumentException when collectionId is null or empty string
     * @throws IndexPersistenceException when collection index can not be retrieved
     */
    public CollectionIndex getCollectionIndex(String collectionId) throws IndexPersistenceException {
        DBIndexPersistenceHelper.validateNotNullOrEmpty(collectionId, "collectionId");

        Connection conn = getConnection();
        beginTransaction(conn);

        CollectionIndex collectionIndex = null;
        try {
            if (isEmpty(conn, SELECT_COLLECTION, new Object[] {collectionId})) {
                return null;
            }
            Set documentIds = getIndexedDocuments(conn, SELECT_COLLECTION_DOCUMENTS, new Object[] {collectionId});
            Map wordMap = new HashMap();

            for (Iterator it = documentIds.iterator(); it.hasNext();) {
                WordSourceId wordSourceId = (WordSourceId) it.next();
                DocumentIndex documentIndex = getDocumentIndexFast(conn, wordSourceId);
                for (Iterator wordIterator = documentIndex.getWords().keySet().iterator(); wordIterator.hasNext();) {

                    Object word = wordIterator.next();
                    Set ids = (Set) wordMap.get(word);
                    if (ids == null) {
                        ids = new HashSet();
                        wordMap.put(word, ids);
                    }
                    ids.add(wordSourceId);
                }
            }
            collectionIndex = new CollectionIndex(null, wordMap, documentIds, collectionId);

        } catch (IndexPersistenceException e) {
            rollbackTransaction(conn);
            throw e;
        }
        commitTransaction(conn);

        return collectionIndex;
    }



    /**
     * <p>Remove document collection index with
     * specified identifier from the persistence.
     *
     * Implementation note:
     * Please consult the CS section 1.4.3.4
     *
     * Throw exception when there is no CollectionIndex with specified identifier.
     * </p>
     *
     * @param collectionId Identifer of document  collection index to remove, should
     * not be null or empty string
     * @throws IllegalArgumentException when collectionId is null or empty string
     * @throws IndexPersistenceException when fails to remove CollectionInde
     * with given identifer
     */
    public void removeCollectionIndex(String collectionId) throws IndexPersistenceException {
        removeCollectionIndexCommon(collectionId);
    }

    /**
     * <p>Update specified CollectionIndex in the persistence.
     * Implementation note:
     * Please consult the CS section 1.4.3.3
     *
     * Throw exception when
     * specified CollectionIndex is not found in the persistence.
     * </p>
     *
     * @param collectionIndex CollectionIndex to update, should not be null
     * @throws IllegalArgumentException if collectionIndex parameter is null
     * @throws IndexPersistenceException if any error happens when updating
     * the collection index in the persistence, this includes the case, when the
     * specified collectionIndex is not found in the persistence
     */
    public void updateCollectionIndex(CollectionIndex collectionIndex) throws IndexPersistenceException {
        updateCollectionIndexCommon(collectionIndex);
    }

    /**
     * <p>Increase the use count value for document index with specified WordSourceId.
     * Use count is the integer value stored in the persistence along with every
     * document index. This value is equal to the number of collections, that contain
     * a particular document.</p>
     *
     * @param wordSourceId WordSourceId of document index use count of which to update,
     * should not be null
     * @throws IllegalArgumentException if wordSourceId is null
     * @throws IndexPersistenceException when fails to increase document index
     * use count in the persistence
     */
    public void increaseDocumentUseCount(WordSourceId wordSourceId) throws IndexPersistenceException {
        updateDocumentUseCount(wordSourceId, 1);
    }

    /**
     * <p>decrease the use count value for document index with specified WordSourceId.
     * Use count is the integer  value stored in the persistence along with every
     * document index.
     * This value is equal to the number of collections, that contain a particular document.
     *
     * Using WordSourceId.hashCode() as the id simply find the corresponding DOCUMENT
     * record and update its USE_COUNT field.
     *
     * If there is no document index with specified WordSourceId, throw an exception
     * </p>
     *
     * @param wordSourceId WordSourceId of document index use count of which to update,
     * should not be null
     * @throws IllegalArgumentException if wordSourceId is null
     * @throws IndexPersistenceException when fails to increase document index
     * use count in the persistence
     */
    public void decreaseDocumentUseCount(WordSourceId wordSourceId) throws IndexPersistenceException {
        updateDocumentUseCount(wordSourceId, -1);
    }

    /**
     * <p>
     * Return set of WordSourceId of documents that have document index persisted.
     *
     * Implementation notes:
     * Simply get all the DOCUMENT records and then for each extract the necessary data
     * to create a WordSourceId instance.
     *
     * Please consult the CS section 1.4.1.1
     *
     * </p>
     *
     * @return set of WordSourceId of documents that have document index persisted.
     * @throws IndexPersistenceException when fails
     */
    public Set getIndexedDocuments() throws IndexPersistenceException {
        return getIndexedDocumentsCommon();
    }

    /**
     * <p>
     * Return the count of how many different groups this document is a member of.
     * </p>
     *
     * @return the count of how many different groups this document is a member of.
     * @param wordSourceId WordSourceId of document index use count of which to update,
     * should not be null
     * @throws IllegalArgumentException if wordSourceId is null
     * @throws IndexPersistenceException when fails to increase document index
     * use count in the persistence
     */
    public int getDocumentUseCount(WordSourceId wordSourceId) throws IndexPersistenceException {
        return getDocumentUseCountCommon(wordSourceId);
    }
}
