/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.document.index.persistence.impl.xml;

import com.topcoder.document.index.CollectionIndex;
import com.topcoder.document.index.DocumentIndex;
import com.topcoder.document.index.persistence.IndexPersistence;
import com.topcoder.document.index.persistence.impl.PersistenceConfigurationException;
import com.topcoder.document.index.wordsource.WordSourceId;

import com.topcoder.util.config.ConfigManager;
import com.topcoder.util.config.UnknownNamespaceException;

import java.util.Set;


/**
 * <p>
 * This is a concrete implementation of the IndexPersistence contract that uses xml files
 * as storage medium. This implementation uses a dual strategy for parsing and utilized
 * SAX parsing for reading data in and DOM parsing for data updates (such as CollectionIndex updates and document
 * use count updates)
 * This implementation allows for creation of arbitrarily large document index files by
 * sequencing (splitting) large files into chunks of file based data. Since it is required
 * that binary data is stored in the xml files for document index a serialized Base64
 * encoded data will be both encoded and decoded by this implementation.
 * Not thread-safe.</p>
 *
 *
 * @author AleaActaEst, TCSDEVELOPER
 * @version 1.0
 */
public class XmlIndexPersistence implements IndexPersistence {
    /**
     * <p>
     * This represent the file name used to represent a document usage as defined by
     * the persistence contract. There is only one such file per xml persistence name space.
     * </p>
     */
    private static final String DOC_USAGE_FILE_NAME = "document_usage.xml";

    /**
     * <p> This represent the file name prefix for all files that represent a Collection
     * Index xml file storage (i.e. a document collecltion file)
     * Each group xml file name will have the following format: doc_group_groupId_.xml
     * For example here is a simple document group xml file for a group with id of 175658:
     * doc_collection_175658.xml
     * </p>
     */
    private static final String DOC_COLLECTION_FILE_NAME_PREFIX = "doc_collection_";

    /**
     * <p> This represent the file name prefix for all files that represent a Document
     * Index xml file storage (i.e. a document index file)
     * Each document index file name will have the following format:
     * doc_index_docId_sequenceid.xml
     *
     * Here is an example of a document index xml file for a document with id of 16527:
     *
     * doc_index_16527_001.xml
     *
     * The sequenceid is used to link files with data that is larger than say a limit
     * of 2G (this can be configured) this
     * means that if we have indexed data that is larger than say 2G we split the file
     * into 2 files with the first having
     * the sequence id of 001 and the second one being obviously 002. This gives the
     * flexibility for persistence of
     * very large content.
     * </p>
     */
    private static final String DOC_INDEX_FILE_NAME_PREFIX = "doc_index_";

    /**
     * <p>Static XML persistence path property config string.</p>
     */
    private static final String XML_PATH_PROPERTY = "XmlPersistencePath";

    /**
     * <p>Static file size property config string.</p>
     */
    private static final String FILE_SIZE_PROPERTY = "FileSizeLimit";

    /**
     * <p>Static file size threshold property config string.</p>
     */
    private static final String FILE_SIZE_THRESHOLD_PROPERTY = "FileSizeThreshold";

    /**
     * <p> This is a file size limit for saving a chunk of indexed data. It is measured
     * in Mbytes. Consider data so large that it cannot fit into a single file due to
     * size limitations for the specific OS. This size limit is used to decide when
     * to start chaining multiple files to store the data.
     * It is initialized from configuration through the constructor and once
     * initialized cannot be changed.
     * Must be a positive integer (i.e. > 0)
     * </p>
     */
    private final int fileSizeLimit;

    /**
     * <p> This is a safety net size in Kbytes that is applied before the  fileSizeLimit
     * is reached. This is basically something of a round off error precaution.
     * It is initialized from configuration through the constructor and once initialized
     * cannot be changed.
     * Must be a non-negative integer (i.e. >= 0)
     * </p>
     */
    private final int fileSizeThreshold;

    /**
     * <p> This is the file path to a directory where all the xml files that make up this
     * persistence session are stored (it is almost like a data base namespace). This
     * must be a valid fully qualified path.
     * It is initialized from configuration through the constructor and once initialized
     * cannot be changed.
     * Cannot be null or an empty string.
     * </p>
     */
    private final String persistenceDirPath;

    /**
     * <p> An simple constructor which will populate the fileSizeLimit, fileSizeThreshold,
     * and persistenceDirPath information from configuration.
     * </p>
     *
     * @param namespace configuration namespace
     * @throws PersistenceConfigurationException if there are configuration issues encountered
     * @throws IllegalArgumentException if namespace param is null or empty.
     */
    public XmlIndexPersistence(String namespace) throws PersistenceConfigurationException {
        validateNotNullOrEmpty(namespace, "namespace");

        try {
            // get properties
            String xmlPath = getConfigString(namespace, XML_PATH_PROPERTY);
            String fileSizeString = getConfigString(namespace, FILE_SIZE_PROPERTY);
            String fileThresholdString = getConfigString(namespace, FILE_SIZE_THRESHOLD_PROPERTY);

            // initialize variables
            this.persistenceDirPath = xmlPath;
            this.fileSizeLimit = Integer.parseInt(fileSizeString);
            this.fileSizeThreshold = Integer.parseInt(fileThresholdString);
        } catch (PersistenceConfigurationException pce) {
            throw pce;
        } catch (Exception e) {
            throw new PersistenceConfigurationException("Configuration is wrong, check inner exception.", e);
        }
    }

    /**
     * <p> An simple convenicence constructor which will populate the fileSizeLimit,
     * fileSizeThreshold, and persistenceDirPath information directly.
     * </p>
     *
     * @param fileSizeLimit This is a file size limit for saving a chunk of indexed data
     * @param fileSizeThreshold This is a safety net size in Kbytes that is applied
     * before the fileSizeLimit is reached
     * @param persistenceDirPath This is the file path to a directory where all the xml
     * files that make up this persistence session are stored
     */
    public XmlIndexPersistence(int fileSizeLimit, int fileSizeThreshold, String persistenceDirPath) {
        this.fileSizeLimit = fileSizeLimit;
        this.fileSizeThreshold = fileSizeThreshold;
        this.persistenceDirPath = persistenceDirPath;
    }

    /**
     * <p>
     * Gets property string from configuration.
     * </p>
     *
     * @param namespace configuration namespace
     * @param propertyName name of property
     * @return property string gotten
     * @throws PersistenceConfigurationException if there are configuration issues encountered
     */
    private String getConfigString(String namespace, String propertyName)
        throws PersistenceConfigurationException {
        ConfigManager configManager = ConfigManager.getInstance();

        String propertyString = null;

        try {
            // get properties
            propertyString = configManager.getString(namespace, propertyName);
        } catch (UnknownNamespaceException une) {
            throw new PersistenceConfigurationException("Specified namespace is wrong.", une);
        }

        if (propertyString.trim().length() == 0) {
            throw new PersistenceConfigurationException(propertyName + " property is required.");
        }

        return propertyString;
    }

    /**
     * <p>Add the given DocumentIndex to the persistence.
     *
     * </p>
     *
     * @param documentIndex WordSource representing the document to be indexed, should
     * not be null
     * @throws IllegalArgumentException when documentIndex is null
     * @throws DocumentIndexPersistenceException when fails to add DocumentIndex to the
     * persistence
     */
    public void addDocumentIndex(DocumentIndex documentIndex) {
        validateNotNull(documentIndex, "documentIndex");
    }

    /**
     * <p>Retrieve a DocumentIndex with the specified WordSourceId from the persistence.
     * Return null, if DocumentIndex with the given WordSourceId is not found in the
     * persistence.
     *
     * Note, than DocumentIndexPersistence exception should NOT be thrown when
     * the DocumentIndex with given WordSourceId is not found in the persistence,
     * but it should be thrown in any other error situations
     *
     * </p>
     *
     *
     *
     * @return retrieved DocumentIndex, or null if document index with given WordSourceId
     * is not found in the persistence
     * @param wordSourceId WordSourceId of document to retrieve, should not be null
     * @throws IllegalArgumentException when wordSourceId is null
     * @throws DocumentIndexPersistenceException when fails to retreive document index
     * with the specified WordSourceId
     */
    public DocumentIndex getDocumentIndex(WordSourceId wordSourceId) {
        return null;
    }

    /**
     * <p>Remove a document index with given WordSourceId from the persistence.
     * When the document index is not found, or function fail to remove it exception is
     * thrown.
     * Function should NOT succeed when document index use count is not zero.
     *
     *  </p>
     *
     * @param wordSourceId WordSourceId of  document index to remove, should not be null
     * @throws IllegalArgumentException when WordSourceId is null
     * @throws DocumentIndexPersistenceException when document index is not found in the
     * peristence, or error happens when trying to remove it
     */
    public void removeDocumentIndex(WordSourceId wordSourceId) {
    }

    /**
     * <p>
     * Store document collection index  in the persistence.
     * </p>
     *
     * @param collectionIndex CollectionIndex to store, should not be null
     * @throws IllegalArgumentException if collectionIndex is null
     * @throws DocumentIndexPersistenceException if fails to create and store document
     * collection index
     */
    public void addCollectionIndex(CollectionIndex collectionIndex) {
    }

    /**
     * <p>Retrieve a document collection index with specified identifier.
     * Return null when collection with specified identifier is not found,
     * throw exception when collection index cannot be retrieved in any
     * other case.
     * </p>
     *
     * @return CollectionIndex with specified identifer
     * @param collectionId identifer of document collection index to retrieve, should not
     * be null or empty string
     * @throws IllegalArgumentException when collectionId is null or empty string
     * @throws DocumentIndexPersistenceException when collection index can not be retrieved
     */
    public CollectionIndex getCollectionIndex(String collectionId) {
        return null;
    }

    /**
     * <p>Remove document collection index with
     * specified identifier from the persistence.
     * </p>
     *
     * @param collectionId Identifer of document  collection index to remove, should
     * not be null or empty string
     * @throws IllegalArgumentException when collectionId is null or empty string
     * @throws DocumentIndexPersistenceException when fails to remove CollectionInde
     * with given identifer
     */
    public void removeCollectionIndex(String collectionId) {
    }

    /**
     * <p>Update specified CollectionIndex in the persistence.
     * </p>
     *
     * @param collectionIndex CollectionIndex to update, should not be null
     * @throws IllegalArgumentException if collectionIndex parameter is null
     * @throws DocumentIndexPersistenceException if any error happens when updating
     * the collection index in the persistence, this includes the case, when the
     * specified collectionIndex is not found in the persistence
     */
    public void updateCollectionIndex(CollectionIndex collectionIndex) {
    }

    /**
     * <p>Increase the use count value for document index with specified WordSourceId.
     * Use count is the integer  value stored in the persistence along with every document
     * index.
     * This value is equal to the number of collections, that contain a particular document.
     * </p>
     *
     * @param wordSourceId WordSourceId of document index use count of which to update,
     * should not be null
     * @throws IllegalArgumentException if wordSourceId is null
     * @throws DocumentIndexPersistenceException when fails to increase document index
     * use count in the persistence
     */
    public void increaseDocumentUseCount(WordSourceId wordSourceId) {
    }

    /**
     * <p>Decrease the use count value for document index with specified WordSourceId.
     * Use count is the integer  value stored in the persistence along with every
     * document index.
     * This value is equal to the number of collections, that contain a particular document.
     * </p>
     *
     * @param wordSourceId WordSourceId of document index use count of which to update,
     * should not be null
     * @throws IllegalArgumentException if wordSourceId is null
     * @throws DocumentIndexPersistenceException when fails to increase document index
     * use count in the persistence
     */
    public void decreaseDocumentUseCount(WordSourceId wordSourceId) {
    }

    /**
     * <p>
     * Return set of WordSourceId of documents that have document index persisted.
     * </p>
     *
     * @return set of WordSourceId of documents that have document index persisted.
     * @throws DocumentIndexPersistenceException when fails
     */
    public Set getIndexedDocuments() {
        return null;
    }

    /**
     * <p>
     * Return the count of how many different groups this document is a member of.
     * </p>
     *
     * @return the count of how many different groups this document is a member of.
     * @param wordSourceId WordSourceId of document index use count of which to update,
     * should not be null
     * @throws IllegalArgumentException if wordSourceId is null
     * @throws DocumentIndexPersistenceException when fails to increase document index
     * use count in the persistence
     */
    public int getDocumentUseCount(WordSourceId wordSourceId) {
        return 0;
    }

    /**
     * <p>
     * Validates the value of a variable with type <code>Object</code>.
     * The value cannot be <code>null</code>.
     * </p>
     *
     * @param value value of the variable.
     * @param name name of the variable.
     *
     * @throws IllegalArgumentException if <code>value</code> is <code>null</code>.
     */
    private void validateNotNull(Object value, String name) {
        if (value == null) {
            throw new IllegalArgumentException(name + " cannot be null.");
        }
    }

    /**
     * <p>
     * Validates the value of a string variable. The value cannot be empty string, but may be <code>null</code>.
     * </p>
     *
     * @param value value of the variable.
     * @param name name of the variable.
     *
     * @throws IllegalArgumentException if <code>value</code> is empty string.
     */
    private void validateNotEmpty(String value, String name) {
        if (value != null) {
            if (value.trim().length() == 0) {
                throw new IllegalArgumentException(name + " cannot be empty string.");
            }
        }
    }

    /**
     * <p>
     * Validates the value of a string variable. The value cannot be <code>null</code> and empty string.
     * </p>
     *
     * @param value value of the variable.
     * @param name name of the variable.
     *
     * @throws IllegalArgumentException if <code>value</code> is null or empty string.
     */
    private void validateNotNullOrEmpty(String value, String name) {
        validateNotNull(value, name);
        validateNotEmpty(value, name);
    }
}
