/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.document.index.persistence.impl.db;

import java.text.Collator;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Set;

import com.topcoder.document.index.CollectionIndex;
import com.topcoder.document.index.DocumentIndex;
import com.topcoder.document.index.persistence.IndexPersistenceException;
import com.topcoder.document.index.wordsource.WordSourceId;
import com.topcoder.util.config.ConfigManager;

import junit.framework.TestCase;

/**
 * Unit tests for SimpleDBIndexPersistenceTests class.
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class SimpleDBIndexPersistenceTests extends TestCase {

    /**
     * The namespace for test.
     */
    private static final String NAMESPACE = "com.topcoder.document.index.persistence.impl.db.unittest";

    /**
     * The configuration file.
     */
    private static final String CONFIG_FILE = "db_simple_config.xml";

    /**
     * The persistence instance to test.
     */
    private SimpleAccessDBIndexPersistence persistence = null;

    /**
     * A DocumentIndex for test.
     */
    private DocumentIndex documentIndex1 = null;

    /**
     * A DocumentIndex for test.
     */
    private DocumentIndex documentIndex2 = null;

    /**
     * A DocumentIndex for test.
     */
    private DocumentIndex documentIndex3 = null;

    /**
     * A DocumentIndex for test.
     */
    private DocumentIndex documentIndex4 = null;

    /**
     * A CollectionIndex for test.
     */
    private CollectionIndex collectionIndex1 = null;

    /**
     * A CollectionIndex for test.
     */
    private CollectionIndex collectionIndex2 = null;

    /**
     * A CollectionIndex for test.
     */
    private CollectionIndex collectionIndex3 = null;

    /**
     * Sets up environment.
     *
     * @throws Exception to JUnit
     */
    protected void setUp() throws Exception {
        ConfigManager configManager = ConfigManager.getInstance();
        if (configManager.existsNamespace(NAMESPACE)) {
            configManager.removeNamespace(NAMESPACE);
        }
        configManager.add(CONFIG_FILE);
        DatabaseHelper.clearTable("word_data");
        DatabaseHelper.clearTable("delimiter");
        DatabaseHelper.clearTable("DOCUMENT_COLLECTION_DOCUMENT_XREF");
        DatabaseHelper.clearTable("document");
        DatabaseHelper.clearTable("DOCUMENT_COLLECTION");

        persistence = new SimpleAccessDBIndexPersistence(NAMESPACE);

        WordSourceId wordSourceId1 = new WordSourceId(
                "StringId1", "java.lang.String", new String[] {" "}, Locale.US);
        documentIndex1 = createDocumentIndex(wordSourceId1, 1);

        WordSourceId wordSourceId2 = new WordSourceId(
                new Integer(1), "java.lang.Integer", new String[] {"123"}, Locale.CHINA);
        documentIndex2 = createDocumentIndex(wordSourceId2, 2);

        WordSourceId wordSourceId3 = new WordSourceId(
                "StringId3", "java.lang.String", new String[] {"\n"}, Locale.US);
        documentIndex3 = createDocumentIndex(wordSourceId3, 3);

        WordSourceId wordSourceId4 = new WordSourceId(
                "StringId4", "java.lang.String", new String[] {"\0\0"}, Locale.US);
        documentIndex4 = createDocumentIndex(wordSourceId4, 4);

        collectionIndex1 = createCollectionIndex("collection1",
                new DocumentIndex[] {documentIndex1, documentIndex2, documentIndex4});

        collectionIndex2 = createCollectionIndex("collection2",
                new DocumentIndex[] {documentIndex3, documentIndex4});

        collectionIndex3 = createCollectionIndex("collection3",
                new DocumentIndex[] {documentIndex2});

    }

    /**
     * Create a DocumentIndex instance for testing.
     *
     * @param wordSourceId the wordSourceId instance.
     * @param n the number of words
     * @return the DocumentIndex instance
     */
    private DocumentIndex createDocumentIndex(WordSourceId wordSourceId, int n) {

        Collator collator = Collator.getInstance(wordSourceId.getSourceLocale());

        Map words = new HashMap();
        for (int i = 0; i < n; ++ i) {
            List positions = new ArrayList();
            for (int j = 0; j <= i ; ++j) {
                positions.add(new Integer(j * 10));
            }
            words.put(collator.getCollationKey("word" + i), positions);
        }
        return new DocumentIndex(wordSourceId, words);
    }

    /**
     * Create a CollectionIndex instance for testing.
     *
     * @param id the CollectionIndex id
     * @param documents the documents in the CollectionIndex
     * @return the CollectionIndex instance
     */
    private CollectionIndex createCollectionIndex(String id, DocumentIndex[] documents) {
        Map wordMap = new HashMap();
        Set documentIds = new HashSet();
        for (int i = 0; i < documents.length; ++i) {

            documentIds.add(documents[i].getWordSourceId());
            for (Iterator it = documents[i].getWords().keySet().iterator(); it.hasNext();) {
                Object word = it.next();
                Set ids = (Set) wordMap.get(word);
                if (ids == null) {
                    ids = new HashSet();
                    wordMap.put(word, ids);
                }
                ids.add(documents[i].getWordSourceId());
            }

        }

        return new CollectionIndex(null, wordMap, documentIds, id);

    }
    /**
     * Clears the config manager and database.
     *
     * @throws Exception to JUnit
     */
    protected void tearDown() throws Exception {

        DatabaseHelper.clearTable("word_data");
        DatabaseHelper.clearTable("delimiter");
        DatabaseHelper.clearTable("DOCUMENT_COLLECTION_DOCUMENT_XREF");
        DatabaseHelper.clearTable("document");
        DatabaseHelper.clearTable("DOCUMENT_COLLECTION");


        ConfigManager configManager = ConfigManager.getInstance();
        if (configManager.existsNamespace(NAMESPACE)) {
            configManager.removeNamespace(NAMESPACE);
        }
    }

    /**
     * Tests the addDocumentIndex method with null.
     * IllegalArgumentException should be thrown
     *
     * @throws Exception to JUnit
     */
    public void testAddDocumentIndex1() throws Exception {
        try {
            persistence.addDocumentIndex(null);
        } catch (IllegalArgumentException iae) {
            // success
        }
    }

    /**
     * Tests the addDocumentIndex method with a valid DocumentIndex instance.
     * The DocumentIndex instance should be added successfully.
     *
     * @throws Exception to JUnit
     */
    public void testAddDocumentIndex2() throws Exception {
        persistence.addDocumentIndex(documentIndex1);

        DocumentIndex documentIndex = persistence.getDocumentIndex(documentIndex1.getWordSourceId());
        checkDocumentIndex(documentIndex1, documentIndex);

    }

    /**
     * Tests the addDocumentIndex method with a valid DocumentIndex instance.
     * The DocumentIndex instance should be added successfully.
     *
     * @throws Exception to JUnit
     */
    public void testAddDocumentIndex3() throws Exception {
        persistence.addDocumentIndex(documentIndex2);

        DocumentIndex documentIndex = persistence.getDocumentIndex(documentIndex2.getWordSourceId());
        checkDocumentIndex(documentIndex2, documentIndex);

    }

    /**
     * Tests the addDocumentIndex method with a valid DocumentIndex instance.
     * The DocumentIndex instance should be added successfully.
     *
     * @throws Exception to JUnit
     */
    public void testAddDocumentIndex4() throws Exception {
        persistence.addDocumentIndex(documentIndex3);

        DocumentIndex documentIndex = persistence.getDocumentIndex(documentIndex3.getWordSourceId());
        checkDocumentIndex(documentIndex3, documentIndex);

    }

    /**
     * Tests the addDocumentIndex method with a valid DocumentIndex instance.
     * The DocumentIndex instance should be added successfully.
     *
     * @throws Exception to JUnit
     */
    public void testAddDocumentIndex5() throws Exception {
        persistence.addDocumentIndex(documentIndex4);

        DocumentIndex documentIndex = persistence.getDocumentIndex(documentIndex4.getWordSourceId());
        checkDocumentIndex(documentIndex4, documentIndex);

    }

    /**
     * Tests the addDocumentIndex method.
     * Adding duplicate DocumentIndex should cause IndexPersistenceException
     *
     * @throws Exception to JUnit
     */
    public void testAddDocumentIndex6() throws Exception {
        persistence.addDocumentIndex(documentIndex1);
        try {
            persistence.addDocumentIndex(documentIndex1);
        } catch (IndexPersistenceException e) {
            // success
        }
    }

    /**
     * Tests the removeDocumentIndex method with null.
     * IllegalArgumentException should be thrown
     *
     * @throws Exception to JUnit
     */
    public void testRemoveDocumentIndex1() throws Exception {
        try {
            persistence.removeDocumentIndex(null);
        } catch (IllegalArgumentException iae) {
            // success
        }
    }

    /**
     * Tests the removeDocumentIndex method with a valid DocumentIndex instance.
     * The DocumentIndex instance should be removed successfully.
     *
     * @throws Exception to JUnit
     */
    public void testRemoveDocumentIndex2() throws Exception {
        persistence.addDocumentIndex(documentIndex1);
        persistence.addDocumentIndex(documentIndex2);
        persistence.removeDocumentIndex(documentIndex1.getWordSourceId());

        assertNull("documentIndex1 should be removed.",
                persistence.getDocumentIndex(documentIndex1.getWordSourceId()));
        assertNotNull("documentIndex2 should not be removed.",
            persistence.getDocumentIndex(documentIndex2.getWordSourceId()));


    }

    /**
     * Tests the removeDocumentIndex method with a valid DocumentIndex instance.
     * The DocumentIndex instance should be removed successfully.
     *
     * @throws Exception to JUnit
     */
    public void testRemoveDocumentIndex3() throws Exception {
        persistence.addDocumentIndex(documentIndex2);
        persistence.removeDocumentIndex(documentIndex2.getWordSourceId());

        assertNull("documentIndex2 should be removed.",
                persistence.getDocumentIndex(documentIndex2.getWordSourceId()));

    }

    /**
     * Tests the removeDocumentIndex method with a valid DocumentIndex instance.
     * The DocumentIndex instance should be removed successfully.
     *
     * @throws Exception to JUnit
     */
    public void testRemoveDocumentIndex4() throws Exception {
        persistence.addDocumentIndex(documentIndex3);
        persistence.removeDocumentIndex(documentIndex3.getWordSourceId());

        assertNull("documentIndex3 should be removed.",
                persistence.getDocumentIndex(documentIndex3.getWordSourceId()));


    }

    /**
     * Tests the removeDocumentIndex method with a valid DocumentIndex instance.
     * The DocumentIndex instance should be removed successfully.
     *
     * @throws Exception to JUnit
     */
    public void testRemoveDocumentIndex5() throws Exception {
        persistence.addDocumentIndex(documentIndex4);
        persistence.removeDocumentIndex(documentIndex4.getWordSourceId());

        assertNull("documentIndex4 should be removed.",
                persistence.getDocumentIndex(documentIndex4.getWordSourceId()));


    }

    /**
     * Tests the removeDocumentIndex method.
     * Removing invalid DocumentIndex should cause IndexPersistenceException
     *
     * @throws Exception to JUnit
     */
    public void testRemoveDocumentIndex6() throws Exception {
        try {
            // IndexPersistenceException should be thrown, since the document does not exist
            persistence.removeDocumentIndex(documentIndex1.getWordSourceId());
        } catch (IndexPersistenceException e) {
            // success
        }
    }

    /**
     * Tests the removeDocumentIndex method.
     * If the use count is not zero, IndexPersistenceException should be thrown.
     *
     * @throws Exception to JUnit
     */
    public void testRemoveDocumentIndex7() throws Exception {
        // add a DocumentIndex with use count 1
        persistence.addDocumentIndex(documentIndex1);
        persistence.increaseDocumentUseCount(documentIndex1.getWordSourceId());

        try {
            // IndexPersistenceException should be thrown, since use count is not zero
            persistence.removeDocumentIndex(documentIndex1.getWordSourceId());
        } catch (IndexPersistenceException e) {
            // success
        }
    }

    /**
     * Tests the getDocumentIndex method with null.
     * IllegalArgumentException should be thrown
     *
     * @throws Exception to JUnit
     */
    public void testGetDocumentIndex1() throws Exception {
        try {
            persistence.getDocumentIndex(null);
        } catch (IllegalArgumentException iae) {
            // success
        }
    }

    /**
     * Tests the getDocumentIndex method with a valid DocumentIndex instance.
     * The DocumentIndex instance should be got successfully.
     *
     * @throws Exception to JUnit
     */
    public void testGetDocumentIndex2() throws Exception {
        persistence.addDocumentIndex(documentIndex1);

        DocumentIndex documentIndex = persistence.getDocumentIndex(documentIndex1.getWordSourceId());
        checkDocumentIndex(documentIndex1, documentIndex);

    }

    /**
     * Tests the getDocumentIndex method with a valid DocumentIndex instance.
     * The DocumentIndex instance should be got successfully.
     *
     * @throws Exception to JUnit
     */
    public void testGetDocumentIndex3() throws Exception {
        persistence.addDocumentIndex(documentIndex2);

        DocumentIndex documentIndex = persistence.getDocumentIndex(documentIndex2.getWordSourceId());
        checkDocumentIndex(documentIndex2, documentIndex);

    }

    /**
     * Tests the getDocumentIndex method with a valid DocumentIndex instance.
     * The DocumentIndex instance should be got successfully.
     *
     * @throws Exception to JUnit
     */
    public void testGetDocumentIndex4() throws Exception {
        persistence.addDocumentIndex(documentIndex3);

        DocumentIndex documentIndex = persistence.getDocumentIndex(documentIndex3.getWordSourceId());
        checkDocumentIndex(documentIndex3, documentIndex);

    }

    /**
     * Tests the getDocumentIndex method with a valid DocumentIndex instance.
     * The DocumentIndex instance should be got successfully.
     *
     * @throws Exception to JUnit
     */
    public void testGetDocumentIndex5() throws Exception {
        persistence.addDocumentIndex(documentIndex4);

        DocumentIndex documentIndex = persistence.getDocumentIndex(documentIndex4.getWordSourceId());
        checkDocumentIndex(documentIndex4, documentIndex);

    }

    /**
     * Tests the getDocumentIndex method.
     * Null should be returned if there is no such DocumentIndex.
     *
     * @throws Exception to JUnit
     */
    public void testGetDocumentIndex6() throws Exception {
        assertNull("null should be returned", persistence.getDocumentIndex(documentIndex1.getWordSourceId()));
    }

    /**
     * Tests the getIndexedDocuments method.
     * No document exists, empty list should be returned;
     *
     * @throws Exception to JUnit
     */
    public void testGetIndexedDocuments1() throws Exception {
        Set documents = persistence.getIndexedDocuments();

        assertTrue("an empty set should be returned", documents.isEmpty());
    }

    /**
     * Tests the getIndexedDocuments method.
     *
     * @throws Exception to JUnit
     */
    public void testGetIndexedDocuments2() throws Exception {
        persistence.addDocumentIndex(documentIndex1);
        persistence.addDocumentIndex(documentIndex2);
        persistence.addDocumentIndex(documentIndex3);
        persistence.addDocumentIndex(documentIndex4);

        Set documents = persistence.getIndexedDocuments();

        assertTrue("documentIndex1 should be in the set", documents.contains(documentIndex1.getWordSourceId()));
        assertTrue("documentIndex2 should be in the set", documents.contains(documentIndex2.getWordSourceId()));
        assertTrue("documentIndex3 should be in the set", documents.contains(documentIndex3.getWordSourceId()));
        assertTrue("documentIndex4 should be in the set", documents.contains(documentIndex4.getWordSourceId()));
    }

    /**
     * Tests the decreaseDocumentUseCount method with null.
     * IllegalArgumentException should be thrown
     *
     * @throws Exception to JUnit
     */
    public void testDecreaseDocumentUseCount1() throws Exception {
        try {
            persistence.decreaseDocumentUseCount(null);
        } catch (IllegalArgumentException iae) {
            // success
        }
    }

    /**
     * Tests the decreaseDocumentUseCount method with a valid DocumentIndex instance.
     * Use count should be updated successfully.
     *
     * @throws Exception to JUnit
     */
    public void testDecreaseDocumentUseCount2() throws Exception {
        persistence.addDocumentIndex(documentIndex1);
        persistence.increaseDocumentUseCount(documentIndex1.getWordSourceId());
        persistence.decreaseDocumentUseCount(documentIndex1.getWordSourceId());

        assertEquals(0, persistence.getDocumentUseCount(documentIndex1.getWordSourceId()));
    }

    /**
     * Tests the decreaseDocumentUseCount method with a valid DocumentIndex instance.
     * Use count should be updated successfully.
     *
     * @throws Exception to JUnit
     */
    public void testDecreaseDocumentUseCount3() throws Exception {
        persistence.addDocumentIndex(documentIndex2);
        persistence.increaseDocumentUseCount(documentIndex2.getWordSourceId());
        persistence.increaseDocumentUseCount(documentIndex2.getWordSourceId());
        persistence.decreaseDocumentUseCount(documentIndex2.getWordSourceId());

        assertEquals(1, persistence.getDocumentUseCount(documentIndex2.getWordSourceId()));
    }

    /**
     * Tests the decreaseDocumentUseCount method.
     * use count is zero, IndexPersistenceException should be thrown.
     *
     * @throws Exception to JUnit
     */
    public void testDecreaseDocumentUseCount4() throws Exception {
        persistence.addDocumentIndex(documentIndex2);
        persistence.increaseDocumentUseCount(documentIndex2.getWordSourceId());

        try {
            // use count is zero, IndexPersistenceException should be thrown.
            persistence.decreaseDocumentUseCount(documentIndex2.getWordSourceId());
        } catch (IndexPersistenceException iae) {
            // success
        }
    }

    /**
     * Tests the decreaseDocumentUseCount method.
     * The document does not exist, IndexPersistenceException should be thrown.
     *
     * @throws Exception to JUnit
     */
    public void testDecreaseDocumentUseCount5() throws Exception {
        try {
            // the document does not exist, IndexPersistenceException should be thrown.
            persistence.decreaseDocumentUseCount(documentIndex3.getWordSourceId());
        } catch (IndexPersistenceException iae) {
            // success
        }
    }

    /**
     * Tests the increaseDocumentUseCount method with null.
     * IllegalArgumentException should be thrown
     *
     * @throws Exception to JUnit
     */
    public void testIncreaseDocumentUseCount1() throws Exception {
        try {
            persistence.increaseDocumentUseCount(null);
        } catch (IllegalArgumentException iae) {
            // success
        }
    }

    /**
     * Tests the increaseDocumentUseCount method with a valid DocumentIndex instance.
     * Use count should be updated successfully.
     *
     * @throws Exception to JUnit
     */
    public void testIncreaseDocumentUseCount2() throws Exception {
        persistence.addDocumentIndex(documentIndex1);
        persistence.increaseDocumentUseCount(documentIndex1.getWordSourceId());
        persistence.increaseDocumentUseCount(documentIndex1.getWordSourceId());

        assertEquals(2, persistence.getDocumentUseCount(documentIndex1.getWordSourceId()));
    }

    /**
     * Tests the increaseDocumentUseCount method with a valid DocumentIndex instance.
     * Use count should be updated successfully.
     *
     * @throws Exception to JUnit
     */
    public void testIncreaseDocumentUseCount3() throws Exception {
        persistence.addDocumentIndex(documentIndex2);
        persistence.increaseDocumentUseCount(documentIndex2.getWordSourceId());

        assertEquals(1, persistence.getDocumentUseCount(documentIndex2.getWordSourceId()));
    }

    /**
     * Tests the increaseDocumentUseCount method.
     * The document does not exist, IndexPersistenceException should be thrown.
     *
     * @throws Exception to JUnit
     */
    public void testIncreaseDocumentUseCount4() throws Exception {
        try {
            // the document does not exist, IndexPersistenceException should be thrown.
            persistence.increaseDocumentUseCount(documentIndex3.getWordSourceId());
        } catch (IndexPersistenceException iae) {
            // success
        }
    }


    /**
     * Tests the getDocumentUseCount method with null.
     * IllegalArgumentException should be thrown
     *
     * @throws Exception to JUnit
     */
    public void testGetDocumentUseCount1() throws Exception {
        try {
            persistence.getDocumentUseCount(null);
        } catch (IllegalArgumentException iae) {
            // success
        }
    }

    /**
     * Tests the getDocumentUseCount method with a valid DocumentIndex instance.
     * Use count should be updated successfully.
     *
     * @throws Exception to JUnit
     */
    public void testGetDocumentUseCount2() throws Exception {
        persistence.addDocumentIndex(documentIndex1);

        assertEquals(0, persistence.getDocumentUseCount(documentIndex1.getWordSourceId()));
    }

    /**
     * Tests the getDocumentUseCount method with a valid DocumentIndex instance.
     * Use count should be updated successfully.
     *
     * @throws Exception to JUnit
     */
    public void testGetDocumentUseCount3() throws Exception {
        persistence.addDocumentIndex(documentIndex2);
        persistence.increaseDocumentUseCount(documentIndex2.getWordSourceId());
        persistence.increaseDocumentUseCount(documentIndex2.getWordSourceId());

        assertEquals(2, persistence.getDocumentUseCount(documentIndex2.getWordSourceId()));
    }



    /**
     * Tests the getDocumentUseCount method.
     * The document does not exist, IndexPersistenceException should be thrown.
     *
     * @throws Exception to JUnit
     */
    public void testGetDocumentUseCount4() throws Exception {
        try {
            // the document does not exist, IndexPersistenceException should be thrown.
            persistence.getDocumentUseCount(documentIndex3.getWordSourceId());
        } catch (IndexPersistenceException iae) {
            // success
        }
    }

    /**
     * Tests the addCollectionIndex method with null.
     * IllegalArgumentException should be thrown
     *
     * @throws Exception to JUnit
     */
    public void testAddCollectionIndex1() throws Exception {
        try {
            persistence.addDocumentIndex(null);
        } catch (IllegalArgumentException iae) {
            // success
        }
    }

    /**
     * Tests the addCollectionIndex method with a valid CollectionIndex instance.
     * The CollectionIndex instance should be added successfully.
     *
     * @throws Exception to JUnit
     */
    public void testAddCollectionIndex2() throws Exception {
        persistence.addDocumentIndex(documentIndex1);
        persistence.addDocumentIndex(documentIndex2);
        persistence.addDocumentIndex(documentIndex3);
        persistence.addDocumentIndex(documentIndex4);

        persistence.addCollectionIndex(collectionIndex1);

        CollectionIndex collectionIndex = persistence.getCollectionIndex(collectionIndex1.getId());
        checkCollectionIndex(collectionIndex1, collectionIndex);

    }

    /**
     * Tests the addCollectionIndex method with 2 valid CollectionIndex instances.
     * The CollectionIndex instances should be added successfully.
     *
     * @throws Exception to JUnit
     */
    public void testAddCollectionIndex3() throws Exception {
        persistence.addDocumentIndex(documentIndex1);
        persistence.addDocumentIndex(documentIndex2);
        persistence.addDocumentIndex(documentIndex3);
        persistence.addDocumentIndex(documentIndex4);

        persistence.addCollectionIndex(collectionIndex1);
        persistence.addCollectionIndex(collectionIndex2);

        CollectionIndex collectionIndex = persistence.getCollectionIndex(collectionIndex1.getId());
        checkCollectionIndex(collectionIndex1, collectionIndex);

    }

    /**
     * Tests the addCollectionIndex method with a valid CollectionIndex instance.
     * The CollectionIndex instance should be added successfully.
     *
     * @throws Exception to JUnit
     */
    public void testAddCollectionIndex4() throws Exception {
        persistence.addDocumentIndex(documentIndex2);

        persistence.addCollectionIndex(collectionIndex3);

        CollectionIndex collectionIndex = persistence.getCollectionIndex(collectionIndex3.getId());
        checkCollectionIndex(collectionIndex3, collectionIndex);

    }

    /**
     * Tests the addCollectionIndex method.
     * Document is missing, IndexPersistenceException should be thrown.
     *
     * @throws Exception to JUnit
     */
    public void testAddCollectionIndex5() throws Exception {
        try {
            persistence.addCollectionIndex(collectionIndex3);
        } catch (IndexPersistenceException e) {
            // success
        }
    }

    /**
     * Tests the addCollectionIndex method.
     * Adding duplicate CollectionIndex should cause IndexPersistenceException
     *
     * @throws Exception to JUnit
     */
    public void testAddCollectionIndex6() throws Exception {
        persistence.addDocumentIndex(documentIndex2);

        persistence.addCollectionIndex(collectionIndex3);

        try {
            persistence.addCollectionIndex(collectionIndex3);
        } catch (IndexPersistenceException e) {
            // success
        }
    }





    /**
     * Checks whether the given DocumentIndex instances are same.
     *
     * @param documentIndex1 the expected DocumentIndex;
     * @param documentIndex2 the DocumentIndex to check
     */
    private void checkDocumentIndex(DocumentIndex documentIndex1, DocumentIndex documentIndex2) {
        assertEquals("WordSourceId is wrong",
                documentIndex1.getWordSourceId(), documentIndex2.getWordSourceId());
        assertEquals("Words is wrong",
                documentIndex1.getWords(), documentIndex2.getWords());
    }

    /**
     * Checks whether the given CollectionIndex instances are same.
     *
     * @param collectionIndex1 the expected DocumentIndex;
     * @param collectionIndex2 the DocumentIndex to check
     */
    private void checkCollectionIndex(CollectionIndex collectionIndex1, CollectionIndex collectionIndex2) {
        assertEquals("owner is wrong",
                collectionIndex1.getOwner(), collectionIndex2.getOwner());
        assertEquals("words is wrong",
                collectionIndex1.getWords(), collectionIndex2.getWords());
        assertEquals("document ids is wrong",
                collectionIndex1.getAllDocumentIds(), collectionIndex2.getAllDocumentIds());
        assertEquals("collection id is wrong",
                collectionIndex1.getId(), collectionIndex2.getId());
    }

}
